import os
import subprocess
from os import listdir

import settings


class Helper:
    def __init__(self):
        self.gnuplot_script = None
        self.name = None

    def generate_eps(self):
        """
        Generate eps figure for the paper.
        """
        # Create figures folder if it does not exist
        if not os.path.exists('figures/'):
            os.makedirs('figures/')
        subprocess.run(
            ["gnuplot " + settings.VISUALIZATION_DIR + self.gnuplot_script],
            shell=True,
            check=True,
        )

    def _check_for_results(self):
        """
        Check for the results in the results folder.
        """
        # Create results folder if it does not exist
        if not os.path.exists('csv/results/'):
            os.makedirs('csv/results/')
        try:
            return (
                settings.RESULTS_DIR
                + sorted([x for x in listdir(settings.RESULTS_DIR) if self.name in x])[-1]
            )
        except IndexError:
            None


def gini(lst):
    """
    Function taken from: https://github.com/oliviaguest/gini
    """
    import numpy as np

    """Calculate the Gini coefficient of a numpy array."""
    # based on bottom eq:
    # http://www.statsdirect.com/help/generatedimages/equations/equation154.svg
    # from:
    # http://www.statsdirect.com/help/default.htm#nonparametric_methods/gini.htm
    array = np.array(lst)
    # All values are treated equally, arrays must be 1d:
    array = array.flatten()
    if np.amin(array) < 0:
        # Values cannot be negative:
        array -= np.amin(array)
    # Values cannot be 0:
    array += 0.0000001
    # Values must be sorted:
    array = np.sort(array)
    # Index per array element:
    index = np.arange(1, array.shape[0] + 1)
    # Number of array elements:
    n = array.shape[0]
    # Gini coefficient:
    return (np.sum((2 * index - n - 1) * array)) / (n * np.sum(array))
