import warnings

from pandas import read_csv


from plotly import graph_objects as go


class ConfidenceIntervals:
    def __init__(self, datasets=None, results=None):
        self.datasets = datasets
        self.results = results

        warnings.filterwarnings("ignore")

    def generate(self):
        """
        Generate the data using the user interval class.
        """
        from identification import UserIntervals

        ui = UserIntervals(self.datasets, self.results)
        ui.generate()
        self.results = ui.results

    def visualize(self, show=True):
        """
        Use plotly to visualize the per task confidence levels.
        """
        graphs = []
        data = read_csv(self.results)
        for feature in ["true_confidence", "false_confidence"]:
            sub = data[(data["segment_interval"] == 8) & (data["number_of_users"] == 7)]
            graphs.append(
                go.Bar(
                    x=sub["task"],
                    y=sub[feature],
                    name=feature,
                    error_y={
                        "type": "data",
                        "array": sub[feature + "_sd"],
                        "visible": True,
                    },
                )
            )

        fig = go.Figure(data=graphs)
        fig.update_layout(
            title="Per task true and false confidence levels",
            xaxis_title="Task",
            xaxis_type="category",
            yaxis_title="Confidence levels",
            yaxis_range=[0, 1],
            barmode="group",
        )
        if show:
            fig.show()
