import subprocess
import warnings
from ast import literal_eval

from pandas import read_csv, unique, DataFrame

import settings

from plotly import graph_objects as go

from helpers.helper import Helper


class SensorInformativeness(Helper):
    def __init__(self, datasets=None, results=None):
        super().__init__()
        self.datasets = datasets
        self.results = results

        warnings.filterwarnings("ignore")

        self.gnuplot_script = "feature_freq.plt"

    def generate(self):
        """
        Generate the results for multiple ml algorithms (import from there).
        """
        from identification import MlComparison

        ml = MlComparison(datasets=self.datasets, results=self.results)
        ml.generate()
        self.results = ml.results

    def visualize(self, show=True):
        """
        Visualize the resulting csv file and generate a gnuplot input csv file.
        """
        df = read_csv(self.results)
        df = df[df["model"] == "RF"]
        TASKS = list(unique(df["task"]))

        results = {"feature": [], "count": []}

        for task in TASKS:
            sub = df[df["task"] == task]
            sub = literal_eval(list(sub["feature_frequency"])[0])

            for feature in [x for x in sub]:

                if feature in results["feature"]:
                    i = results["feature"].index(feature)
                    results["count"][i] += 1
                else:
                    results["feature"].append(feature)
                    results["count"].append(1)

        results["count"] = [x / 3 for x in results["count"]]
        df = DataFrame(results)
        df = df.sort_values(by=["count", "feature"], ascending=False)

        fig = go.Figure(data=[go.Bar(x=df["feature"], y=df["count"])])
        fig.update_layout(
            title="Feature frequency",
            xaxis_title="Feature",
            xaxis_type="category",
            yaxis_title="Frequency",
        )
        if show:
            fig.show()
        df.to_csv(settings.VISUALIZATION_DIR + "feature_freq.csv", index=False)
