from os import mkdir, remove
from os.path import isdir
from zipfile import ZipFile

from IPython.core.display import clear_output
from gdown import download

import settings


class FetchDataHandler:
    def __init__(self, dataset_url=settings.DATASET_URL, data_dir=settings.DATA_DIR):
        self.dataset_url = dataset_url
        self.dataset_file = "dataset.zip"
        self.data_dir = data_dir

    def fetch_data(self):
        """
        Download the data sets from the DATA_URLS dict defined in the settings.
        """
        self._check_dir()

        download(self.dataset_url, self.data_dir + self.dataset_file, quiet=False)
        print("Data downloaded successfully.")

        self._unzip_data()
        self._remove_zip()

    def _check_dir(self):
        """
        Check if data directory exists and create it if it doesn't.
        """
        if not isdir(self.data_dir):
            mkdir(self.data_dir)

    def _unzip_data(self):
        """
        Unzip the downloaded zip file in the right directory.
        """
        print("Unzipping data...", end="")
        zip = ZipFile(self.data_dir + self.dataset_file)
        zip.extractall(self.data_dir)
        print("DONE")

    def _remove_zip(self):
        """
        After the dataset is extracted, remove the downloaded zip file.
        """
        print("Removing zip file...", end="")
        remove(self.data_dir + self.dataset_file)
        print("DONE")
