from datetime import timedelta, datetime

import settings
from processing.dataset_handler import DataSetHandler


class SeanceHandler:
    def __init__(self, task_number: int):
        """
        Class that provides function that are (per task) seance related.
        """
        self.task_number = task_number

        self.data = DataSetHandler(
            settings.DATA_DIR + "task_{}.csv".format(str(task_number).zfill(2))
        )
        # self.data = DataSetHandler(
        #     settings.DATA_DIR
        #     + "task_{}_sample_seance.csv".format(str(task_number).zfill(2))
        # )
        self.data.import_data()

    def get_valid_seances(self):
        """
        Subset the dataframe, based on the seance validity check.
        """
        invalid_users = []

        for seance_id in self.data.seances:
            if not self.is_valid_seance(seance_id):
                user_id = [x[0] for x in self.data.user_seances if x[1] == seance_id][0]
                print("Excluding user {}.".format(user_id))
                invalid_users.append(user_id)
        self.data.subset_data(
            [x[1] for x in self.data.user_seances if x[0] not in invalid_users]
        )

    def is_valid_seance(self, seance_id):
        """
        Check if the seance is valid.
        """
        print("Checking seance {}.".format(seance_id))
        seance = self.data.get_data_for_seance(seance_id)
        timestamps = sorted(seance["timestamp"])
        seance_start = timestamps[0]
        seance_end = timestamps[-1]

        # Seances shorter than a minute are automatically invalid
        if seance_end - seance_start < timedelta(seconds=60 * 3):
            print(
                "Removing seance {}, because of insufficient length.".format(seance_id)
            )
            return False

        # If only one seance per user per task, those are invalid as well
        if not self.data.is_2_seances(seance_id=seance_id):
            print(
                "Removing seance {}, because of a lack of a partner seance.".format(
                    seance_id
                )
            )
            return False

        for sensor in self.data.sensors:
            if not self._is_valid_signal(seance, sensor):
                return False
        print("Seance OK")
        return True

    def _is_valid_signal(self, seance, sensor):
        """
        Check if the signal is valid.
        """
        if "pir" in sensor or "hall" in sensor:
            return True
        records = seance[seance["sensor"] == sensor].sort_values(by="timestamp")

        # Check if there is data for sensor, if not, exclude.
        if records.empty:
            print("Removing seance, because of no data for {}.".format(sensor))
            return False

        return True
